@@ -855,6 +855,32 @@ describe("Terminal:", function()
         assert.is.near(1, timing, 0.5)       -- this also works for MacOS in CI
       end)
 
+
+      it("calls flseep to execute the sleep", function()
+        setbuffer("")
+
+        local sleep_called = false
+        local mysleep = function()
+          sleep_called = true
+        end
+
+        system.readkey(0.01, mysleep)
+        assert.is_true(sleep_called)
+      end)
+
+
+      it("returns errors by fsleep", function()
+        setbuffer("")
+
+        local mysleep = function()
+          return nil, "boom!"
+        end
+
+        local ok, err = system.readkey(1, mysleep)
+        assert.is.falsy(ok)
+        assert.equals("boom!", err)
+      end)
+
     end)
 
 
@@ -876,6 +902,14 @@ describe("Terminal:", function()
       end)
 
 
+      it("reads a control byte for single-byte control characters", function()
+        setbuffer("\000\031\127")
+        assert.are.same({"\000", "ctrl"}, {system.readansi(0)})
+        assert.are.same({"\031", "ctrl"}, {system.readansi(0)})
+        assert.are.same({"\127", "ctrl"}, {system.readansi(0)})
+      end)
+
+
       it("reads a multi-byte characters one at a time", function()
         setbuffer(string.char(226, 130, 172) ..        -- "€"   single
                   string.char(240, 159, 154, 128))     -- "🚀"  double
@@ -902,10 +936,31 @@ describe("Terminal:", function()
       end)
 
 
+      it("reads ANSI escape sequences, just an '<esc>O' (<alt><O> key press)", function()
+        setbuffer("\27O")
+        assert.are.same({"\27O", "ansi"}, {system.readansi(0)})
+      end)
+
+
+      it("reads <alt>+key key presses; <esc>+<key>", function()
+        setbuffer("\27a\27b\27c\27d")
+        assert.are.same({"\27a", "ansi"}, {system.readansi(0)})
+        assert.are.same({"\27b", "ansi"}, {system.readansi(0)})
+        assert.are.same({"\27c", "ansi"}, {system.readansi(0)})
+        assert.are.same({"\27d", "ansi"}, {system.readansi(0)})
+      end)
+
+
+      it("reads <ctrl><alt>[ key press; <esc>+<esc>", function()
+        setbuffer("\27\27\27\27")
+        assert.are.same({"\27\27", "ansi"}, {system.readansi(0)})
+        assert.are.same({"\27\27", "ansi"}, {system.readansi(0)})
+      end)
+
+
       it("returns a single <esc> character if no sequence is found", function()
-        setbuffer("\27\27[A")
-        assert.are.same({"\27", "char"}, {system.readansi(0)})
-        assert.are.same({"\27[A", "ansi"}, {system.readansi(0)})
+        setbuffer("\27")
+        assert.are.same({"\27", "ctrl"}, {system.readansi(0)})
       end)
 
 
@@ -919,6 +974,22 @@ describe("Terminal:", function()
         assert.is.near(1, timing, 0.5)       -- this also works for MacOS in CI
       end)
 
+
+      it("incomplete ANSI sequences will be completed on next call", function()
+        setbuffer("\27[")
+        assert.are.same({nil, "timeout", "\27["}, {system.readansi(0)})
+        setbuffer("A")
+        assert.are.same({"\27[A", "ansi"}, {system.readansi(0)})
+      end)
+
+
+      it("incomplete UTF8 sequences will be completed on next call", function()
+        setbuffer(string.char(240, 159))
+        assert.are.same({nil, "timeout", string.char(240, 159)}, {system.readansi(0)})
+        setbuffer(string.char(154, 128))
+        assert.are.same({"🚀", "char"}, {system.readansi(0)})
+      end)
+
     end)
 
   end)
