@@ -384,6 +384,7 @@ See [setconsolemode documentation](https://learn.microsoft.com/en-us/windows/con
 @treturn[1] boolean `true` on success
 @treturn[2] nil
 @treturn[2] string error message
+@within Terminal_Windows
 @usage
 local system = require('system')
 system.listconsoleflags(io.stdout) -- List all the available flags and their current status
@@ -427,13 +428,12 @@ input flags (for use with `io.stdin`) and `COF` are the output flags (for use wi
 _Note_: See [setconsolemode documentation](https://learn.microsoft.com/en-us/windows/console/setconsolemode)
 for more information on the flags.
 
-
-
 @function getconsoleflags
 @tparam file file file handle to operate on, one of `io.stdin`, `io.stdout`, `io.stderr`
 @treturn[1] bitflags the current console flags.
 @treturn[2] nil
 @treturn[2] string error message
+@within Terminal_Windows
 @usage
 local system = require('system')
 
@@ -497,6 +497,7 @@ The terminal attributes is a table with the following fields:
 @treturn[2] string error message
 @treturn[2] int errnum
 @return error message if failed
+@within Terminal_Posix
 @usage
 local system = require('system')
 
@@ -591,6 +592,7 @@ _Note_: only `iflag`, `oflag`, and `lflag` are supported at the moment. The othe
 @return[2] nil
 @treturn[2] string error message
 @treturn[2] int errnum
+@within Terminal_Posix
 @usage
 local system = require('system')
 
@@ -649,8 +651,105 @@ static int lst_tcsetattr(lua_State *L)
 
 
 
+#ifndef _WIN32
+/*
+reopen FDs for independent file descriptions.
+*/
+static void reopen_fd(lua_State *L, int fd, int flags) {
+    char path[64];
+    int newfd = -1;
+
+    if (fd != STDOUT_FILENO && fd != STDERR_FILENO) {
+        luaL_error(L, "Invalid file descriptor: %d. Only stdout (1) and stderr (2) are supported.", fd);
+    }
+
+    const char *fallback_path = (fd == STDOUT_FILENO) ? "/dev/stdout" :
+                                (fd == STDERR_FILENO) ? "/dev/stderr" : NULL;
+
+    // If fd is a terminal, reopen its actual device (e.g. /dev/ttys003)
+    // Works on all POSIX platforms that have terminals (macOS, Linux, BSD, etc.)
+    if (isatty(fd)) {
+        const char *tty = ttyname(fd);
+        if (tty) {
+            newfd = open(tty, flags);
+        }
+    }
+
+    if (newfd < 0) {
+        // For non-tty: try /dev/fd/N — POSIX-compliant and standard on macOS, Linux, BSD.
+        // This gives a new file description even if the target is a file or pipe.
+        snprintf(path, sizeof(path), "/dev/fd/%d", fd);
+        newfd = open(path, flags);
+    }
+
+    if (newfd < 0) {
+        // Fallback: for platforms/environments where /dev/fd/N doesn't exist.
+        // /dev/stdout and /dev/stderr are standard on Linux, but may not create new descriptions.
+        if (fallback_path) {
+            newfd = open(fallback_path, flags);
+        }
+    }
+
+    if (newfd < 0) {
+        // All attempts failed — raise error with detailed info (call will not return)
+        luaL_error(L, "Failed to reopen fd %d: tried ttyname(), /dev/fd/%d, and fallback %s: %s",
+                    fd, fd, fallback_path ? fallback_path : "(none)", strerror(errno));
+    }
+
+    // Replace the original fd with the new one
+    if (dup2(newfd, fd) < 0) {
+        close(newfd);
+        luaL_error(L, "dup2 failed for fd %d: %s", fd, strerror(errno));
+    }
+
+    close(newfd); // Close the new fd, as dup2 has replaced the original fd with it
+}
+#endif
+
+
+
+/***
+Creates new file descriptions for `stdout` and `stderr`.
+Even if the file descriptors are unique, they still might point to the same
+file description, and hence share settings like `O_NONBLOCK`. This means that
+if one of them is set to non-blocking, the other will be as well. This can
+lead to unexpected behavior.
+
+This function is used to detach `stdout` and `stderr` from the original
+file descriptions, and create new file descriptions for them. This allows
+independent control of flags (e.g., `O_NONBLOCK`) on `stdout` and `stderr`,
+avoiding shared side effects.
+
+Does not modify `stdin` (fd 0), and does nothing on Windows.
+@function detachfds
+@return boolean `true` on success, or throws an error on failure.
+@within Terminal_Posix
+@see setnonblock
+*/
+static int lst_detachfds(lua_State *L) {
+    static int already_detached = 0; // detaching is once per process(not per thread or Lua state)
+    if (already_detached) {
+        lua_pushnil(L);
+        lua_pushliteral(L, "stdout and stderr already detached");
+        return 1;
+    }
+    already_detached = 1;
+
+#ifndef _WIN32
+    // Reopen stdout and stderr with new file descriptions
+    reopen_fd(L, STDOUT_FILENO, O_WRONLY);
+    reopen_fd(L, STDERR_FILENO, O_WRONLY);
+#endif
+
+    lua_pushboolean(L, 1);
+    return 1;
+}
+
+
+
 /***
 Enables or disables non-blocking mode for a file (Posix).
+Check `detachfds` in case there are shared file descriptions.
 @function setnonblock
 @tparam file fd file handle to operate on, one of `io.stdin`, `io.stdout`, `io.stderr`
 @tparam boolean make_non_block a truthy value will enable non-blocking mode, a falsy value will disable it.
@@ -658,9 +757,12 @@ Enables or disables non-blocking mode for a file (Posix).
 @treturn[2] nil
 @treturn[2] string error message
 @treturn[2] int errnum
+@within Terminal_Posix
 @see getnonblock
+@see detachfds
 @usage
 local sys = require('system')
+sys.detachfds() -- detach stdout and stderr, so only stdin becomes non-blocking
 
 -- set io.stdin to non-blocking mode
 local old_setting = sys.getnonblock(io.stdin)
@@ -717,6 +819,8 @@ Gets non-blocking mode status for a file (Posix).
 @treturn[2] nil
 @treturn[2] string error message
 @treturn[2] int errnum
+@within Terminal_Posix
+@see setnonblock
 */
 static int lst_getnonblock(lua_State *L)
 {
@@ -783,6 +887,7 @@ sequences will be buffered internally and returned one byte at a time.
 @treturn[3] nil on error
 @treturn[3] string error message
 @treturn[3] int errnum (on posix)
+@within Terminal_Input
 */
 static int lst_readkey(lua_State *L) {
 #ifdef _WIN32
@@ -803,7 +908,18 @@ static int lst_readkey(lua_State *L) {
     }
 
     wchar_t wc = _getwch();
-    // printf("----\nread wchar_t: %x\n", wc);
+    // printf("----\nread wchar_t: %x\n", (unsigned int)wc);
+
+    if (wc == 0x00 || wc == 0xE0) {
+        // printf("Ignoring scan code: %x\n", (unsigned int)wc);
+        // On Windows, especially with key-repeat, we can get native scancodes even if we've set the console
+        // to ANSI mode. Something, something, terminal not keeping up... These are not valid wide characters,
+        // so we ignore them. Scan codes start with either 0x00 or 0xE0, and have a follow up byte, which we all ignore.
+        // These codes are unique, so we do not risk dropping valid data.
+        _getwch(); // Discard 2nd half of the scan code
+        return 0;
+    }
+
     if (wc == WEOF) {
         lua_pushnil(L);
         lua_pushliteral(L, "read error");
@@ -824,7 +940,7 @@ static int lst_readkey(lua_State *L) {
             }
 
             wchar_t wc2 = _getwch();
-            // printf("read wchar_t 2: %x\n", wc2);
+            // printf("read wchar_t 2: %x\n", (unsigned int)wc2);
             if (wc2 == WEOF) {
                 lua_pushnil(L);
                 lua_pushliteral(L, "read error");
@@ -972,6 +1088,7 @@ Get the width of a utf8 character for terminal display.
 @treturn[1] int the display width in columns of the first character in the string (0 for an empty string)
 @treturn[2] nil
 @treturn[2] string error message
+@within Terminal_UTF-8
 */
 int lst_utf8cwidth(lua_State *L) {
     int width = 0;
@@ -1033,6 +1150,7 @@ Get the width of a utf8 string for terminal display.
 @treturn[1] int the display width of the string in columns (0 for an empty string)
 @treturn[2] nil
 @treturn[2] string error message
+@within Terminal_UTF-8
 */
 int lst_utf8swidth(lua_State *L) {
     const char *utf8_str;
@@ -1083,6 +1201,7 @@ int lst_utf8swidth(lua_State *L) {
 Gets the current console code page (Windows).
 @function getconsolecp
 @treturn[1] int the current code page (always 65001 on Posix systems)
+@within Terminal_UTF-8
 */
 static int lst_getconsolecp(lua_State *L) {
     unsigned int cp = 65001;
@@ -1100,6 +1219,7 @@ Sets the current console code page (Windows).
 @function setconsolecp
 @tparam int cp the code page to set, use `system.CODEPAGE_UTF8` (65001) for UTF-8
 @treturn[1] bool `true` on success (always `true` on Posix systems)
+@within Terminal_UTF-8
 */
 static int lst_setconsolecp(lua_State *L) {
     unsigned int cp = (unsigned int)luaL_checkinteger(L, 1);
@@ -1117,6 +1237,7 @@ static int lst_setconsolecp(lua_State *L) {
 Gets the current console output code page (Windows).
 @function getconsoleoutputcp
 @treturn[1] int the current code page (always 65001 on Posix systems)
+@within Terminal_UTF-8
 */
 static int lst_getconsoleoutputcp(lua_State *L) {
     unsigned int cp = 65001;
@@ -1134,6 +1255,7 @@ Sets the current console output code page (Windows).
 @function setconsoleoutputcp
 @tparam int cp the code page to set, use `system.CODEPAGE_UTF8` (65001) for UTF-8
 @treturn[1] bool `true` on success (always `true` on Posix systems)
+@within Terminal_UTF-8
 */
 static int lst_setconsoleoutputcp(lua_State *L) {
     unsigned int cp = (unsigned int)luaL_checkinteger(L, 1);
@@ -1157,6 +1279,7 @@ static luaL_Reg func[] = {
     { "setconsoleflags", lst_setconsoleflags },
     { "tcgetattr", lst_tcgetattr },
     { "tcsetattr", lst_tcsetattr },
+    { "detachfds", lst_detachfds },
     { "getnonblock", lst_getnonblock },
     { "setnonblock", lst_setnonblock },
     { "_readkey", lst_readkey },
